在决赛第一阶段运行评测程序时，出现非常多的问题，导致无法正常启动，连 main 函数都进不去。这些问题主要涉及加载可执行文件时用户栈相关变量参数的放置、页表数据的复制、`tp` 寄存器的使用等，这里进行统一记录。

# LoadPageFault

在刚开始运行时，会出现 LoadPageFault 错误，原因是访问的地址不在正常的地址空间内，其访问的地址直接超过了用户栈的最高位，而这是不可能的。改变用户栈的地址时，产生该错误的指令不变，但访问的地址不同，但其相对用户栈最高位的偏移不变。

<center><img src="./img/img7.jpg" style="zoom:80%;" /></center>

<center><img src="./img/img8.jpg" style="zoom:80%;" /></center>

通过调试定位出出错的指令，但操作系统并不会对指令的运行顺序等做出改变，因此只能是指令在运行的过程中的某些值发生异常，并不是正常的预期结果。由于不知道哪些值或者寄存器出现异常，所以调试跟踪很难确定出现错误的原因。不过偶然看到程序的源码后发现在进行 `libc` 库初始化时对栈的设置进行了假设，而这就是错误的原因了。

<center><img src="./img/img9.jpg" style="zoom:80%;" /></center>

# 程序加载用户栈设置

在原本的 rcore 操作系统中，用户栈初始化时格式如下。可以看到在原来的设计中，并没有添加对 envp 变量的设置，且变量放置的格式也不正确。因此后来打算直接采用 linux 0.11 的设计，这时虽然成功运行了原来出错的指令，但又导致了新的 LoadPageFault 错误。

<center><img src="./img/img10.jpg" style="zoom:80%;" /></center>

再次考虑相同的原因，查到源码中存在另一处对栈的假设，如图所示。

<center><img src="./img/img11.jpg" style="zoom:80%;" /></center>

这里多了一个叫作 `辅助向量` 的概念，由此找到新的栈格式，具体可参考后面的参考资料。最后在系统中添加了对辅助向量的有关结构（`aux.rs`），其可能的参数也是由参考资料合并来的，没找到具体的文档，所以可能有缺漏。最终设计的栈格式如下。

<center><img src="./img/img12.jpg" style="zoom:50%;" /></center>

# tp 寄存器使用

在改变栈格式后，程序继续运行，此时出现另一个错误，在获得 hartid 的时候取得错误的值，导致索引具体的 `processor` 或 `manager` 结构时出错，而 hartid 是通过读取 `tp` 寄存器得到的，因此在程序运行过程中改变了 `tp` 寄存器。原本在 rcore 中，假设了 `tp` 寄存器不被使用，所以将 `tp` 寄存器用作 CPU 核 id，但在运行的可执行文件中，使用了 `tp` 寄存器，因此原先的假设错误。所以我们在 trap 上下文中对 `tp` 寄存器进行了保存，一旦进入内核，`tp` 寄存器则表示核 id，回到用户态时则恢复 `tp` 寄存器的值，同时注意在 `trap.S` 的上下文切换中也要更改相应的代码。

# 总结

上述问题都是由于移植的操作系统 rcore 本身的缺陷导致的，而实际的评测程序是由 `musl` 编译的，`musl` 是 `libc` 库的一个开源实现，在实现的时候符合相应的标准，而且很多标准可能比较新，导致与之前的不一样，因此在实现的时候应尽可能与标准一致。因此在加载的时候多了很多步骤，这些步骤是现在加载所必须的，也就是完完全全现在实际应用的，而在初赛的时候进行了简化，并没有这些要求。总之，决赛的要求比较高，要求能真实地加载运行实际的程序，完成相应的功能。

# 参考资料

[About ELF Auxiliary Vectors (manugarg.com)](http://articles.manugarg.com/aboutelfauxiliaryvectors.html)（注意这里是 32 位的，而实现的系统是 64 位的）

https://github.com/torvalds/linux/blob/master/fs/binfmt_elf.c