use spin::{Mutex, MutexGuard};

use super::{SignalFlags, MAX_SIG};

/// Action for a signal
#[repr(C)]
#[derive(Debug, Clone, Copy)]
pub struct SignalAction {
    pub handler: usize,
    pub flags: usize,
    pub restorer: usize,
    pub mask: SignalFlags,
}

impl Default for SignalAction {
    fn default() -> Self {
        Self {
            handler: 0,
            flags: 0,
            restorer: 0,
            mask: SignalFlags::from_bits(0).unwrap(),
        }
    }
}

pub struct SignalActions {
    inner: Mutex<SignalActionsInner>,
}

#[derive(Clone)]
pub struct SignalActionsInner {
    pub table: [SignalAction; MAX_SIG + 1],
}

impl SignalActions {
    pub fn inner_exclusive_access(&self) -> MutexGuard<'_, SignalActionsInner> {
        self.inner.lock()
    }

    pub fn clone(&self) -> Self {
        let inner = self.inner_exclusive_access();
        Self {
            inner: Mutex::new(SignalActionsInner {
                table: inner.table.clone(),
            }),
        }
    }
}

impl Default for SignalActions {
    fn default() -> Self {
        Self {
            inner: Mutex::new(SignalActionsInner {
                table: [SignalAction::default(); MAX_SIG + 1],
            }),
        }
    }
}
